﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace Deleter
{
    public class AppParams
    {

        /// <summary>
        /// List of params and constants to index them by
        /// </summary>
        private static class PARAM_STRINGS
        {
            public static List<string> _Params = new List<string>() { "cutoff:", "output:", "guid", "folders", "numeric" };
            public const int CUTOFF = 0;
            public const int OUTPUT = 1;
            public const int GUID = 2;
            public const int FOLDERS = 3;
            public const int NUMERIC = 4;
        }

        public string ErrMsg { get; set; }
        public bool GuidMode { get; set; }
        public bool FolderMode { get; set; }
        public bool NumericMode { get; set; }
        public string SearchDir { get; set; }
        public DateTime? Cutoff { get; set; }
        public string OutputPath { get; set; }

        public bool Validate( string[] args )
        {
            SetGuidMode( args );
            SetFolderMode( args );
            SetNumericMode( args );
            return ValidSearchDir( args ) && ValidCutoff( args ) && ValidOutPath( args );
        }

        private void SetNumericMode( string[] args )
        {
            string numericParam = args.Where( a => a.Equals( PARAM_STRINGS._Params[PARAM_STRINGS.NUMERIC] ) ).FirstOrDefault();
            NumericMode = numericParam != null;
        }

        private void SetFolderMode( string[] args )
        {
            string folderParam = args.Where( a => a.Equals( PARAM_STRINGS._Params[PARAM_STRINGS.FOLDERS] ) ).FirstOrDefault();
            FolderMode = folderParam != null;
        }

        private void SetGuidMode( string[] args )
        {
            string guidParam = args.Where( a => a.Equals( PARAM_STRINGS._Params[PARAM_STRINGS.GUID] ) ).FirstOrDefault();
            GuidMode = guidParam != null;
        }

        private bool ValidOutPath( string[] args )
        {
            var outpathParams = args.Where( a => a.StartsWith( PARAM_STRINGS._Params[PARAM_STRINGS.OUTPUT] ) );

            if ( outpathParams.Count() == 0 )
            {
                OutputPath = null;
                return true;
            }
            else if ( outpathParams.Count() > 1 )
            {
                ErrMsg = "Duplicate output params.";
                return false;
            }
            else
            {
                OutputPath = outpathParams.First().Substring( PARAM_STRINGS._Params[PARAM_STRINGS.OUTPUT].Length );
                try
                {
                    if ( File.Exists( OutputPath ) )
                    {
                        File.Delete( OutputPath );
                    }
                    return true;
                }
                catch ( Exception )
                {
                    ErrMsg = "Error creating or deleting previous output file";
                    return false;
                }
            }
        }

        private bool ValidCutoff( string[] args )
        {
            var result = false;

            var cutoffParams = args.Where( a => a.StartsWith( PARAM_STRINGS._Params[PARAM_STRINGS.CUTOFF] ) );

            if ( cutoffParams.Count() == 0 )
            {
                Cutoff = null;
                result = true;
            }
            else if ( cutoffParams.Count() > 1 )
            {
                ErrMsg = "Duplicate cutoff params.";
            }
            else
            {
                int test;
                result = int.TryParse( cutoffParams.First().Substring( PARAM_STRINGS._Params[PARAM_STRINGS.CUTOFF].Length ), out test );
                if ( result )
                {
                    if ( test == 0 )
                    {
                        Cutoff = DateTime.Now;  // don't cutoff earlier today at midnight; delete anything created before now.
                    }
                    else
                    {
                        Cutoff = DateTime.Now.Date.AddDays( -test );
                    }
                }
                else
                {
                    ErrMsg = "Invalid cutoff param.";
                }
            }

            return result;
        }

        private bool ValidSearchDir( string[] args )
        {
            var result = false;

            /// Look for params that don't match any of the keywords in PARAM_STRINGS.  There should only be one.
            var searchPathParams = args.Where( a =>
            {
                var match = false;
                PARAM_STRINGS._Params.ForEach( p =>
                {
                    if ( a.StartsWith( p ) )
                    {
                        match = true;
                    }
                } );
                return !match;
            } );

            if ( searchPathParams.Count() == 1 )
            {
                SearchDir = searchPathParams.First();
                result = Directory.Exists( SearchDir );
                if ( !result )
                {
                    ErrMsg = "Search dir doesn't exist";
                }
            }
            else
            {
                ErrMsg = "One or more params don't match input requirements.";
            }

            return result;
        }

    }
}
